## spatstat.geom NAMESPACE file

## ................  Import packages ..................

import(stats,graphics,grDevices,utils,methods)
import(spatstat.utils,spatstat.data,spatstat.univar)
import(polyclip)
importFrom(deldir,
           deldir,duplicatedxy,tile.list)

## ................ Load dynamic library ..............
##      (native routines are registered in init.c)
##      (entry points are symbols with prefix "SG_")

useDynLib(spatstat.geom, .registration=TRUE, .fixes="SG_")

## //////////  DO NOT EDIT THE FOLLOWING  ///////////////
## ////////  it is generated automatically  /////////////




# .................................................. 
# Automatically-generated list of documented objects 
# .................................................. 
export("$.hyperframe")
export("$<-.hyperframe")
export("acedist.noshow")
export("acedist.show")
export("add.texture")
export("affine")
export("affine.diagramobj")
export("affine.distfun")
export("affine.im") 
export("affine.layered")
export("affine.owin") 
export("affine.ppp") 
export("affine.psp")
export("affine.tess") 
export("affinexy")
export("affinexypolygon")
export("allElementsIdentical")
export("angles.psp")
export("anycrossing.psp")
export("anyDuplicated.ppp")
export("anyDuplicated.ppx")
export("anylapply")
export("[.anylist")
export("[<-.anylist")
export("anylist")
export("anyNA.im")
export("append.psp")
export("applynbd")
export("applyPolyclipArgs")
export("applytolayers")
export("area")
export("area.default")
export("areaGain")
export("areaGain.diri")       
export("areaGain.grid")       
export("areaLoss")
export("areaLoss.diri")       
export("areaLoss.grid")
export("areaLoss.poly")
export("area.owin")
export("as.anylist")
export("as.array.im")
export("as.box3")
export("as.boxx")
export("as.character.unitname")
export("as.colourmap")
export("as.colourmap.colourmap")
export("as.colourmap.symbolmap")
export("as.data.frame.hyperframe")
export("as.data.frame.im")
export("as.data.frame.NAobject")
export("as.data.frame.owin")
export("as.data.frame.ppp")
export("as.data.frame.ppplist")
export("as.data.frame.ppx")
export("as.data.frame.psp")
export("as.data.frame.tess")
export("as.double.im")
export("as.function.im")
export("as.function.owin")
export("as.function.tess")
export("as.hyperframe")
export("as.hyperframe.anylist")
export("as.hyperframe.data.frame")
export("as.hyperframe.default")
export("as.hyperframe.hyperframe")
export("as.hyperframe.listof")
export("as.hyperframe.ppx")
export("as.im")
export("as.im.data.frame")
export("as.im.default")
export("as.im.distfun")
export("as.im.expression")
export("as.im.function")
export("as.im.funxy")
export("as.im.im")
export("as.imlist")
export("as.im.matrix")
export("as.im.NAobject")
export("as.im.nnfun")
export("as.im.owin")
export("as.im.ppp")
export("as.im.tess")
export("as.layered")
export("as.layered.default")
export("as.layered.listof")
export("as.layered.ppp")
export("as.layered.solist")
export("as.layered.splitppp")
export("as.linimlist")
export("as.list.hyperframe")
export("as.listof")
export("as.mask")
export("AsMaskInternal")
export("as.mask.psp")
export("as.matrix.im")
export("as.matrix.owin")
export("as.matrix.ppx")
export("as.owin")
export("as.owin.boxx")
export("as.owin.data.frame")
export("as.owin.default")
export("as.owin.distfun")
export("as.owin.funxy")
export("as.owin.im")
export("as.owin.layered")
export("as.owin.NAobject")
export("as.owin.nnfun")
export("as.owin.owin")
export("as.owin.ppp")
export("as.owin.psp")
export("as.owin.quad")
export("as.owin.quadratcount")
export("as.owin.tess")
export("as.polygonal")
export("as.ppp")
export("as.ppp.data.frame")
export("as.ppp.default")
export("as.ppplist")
export("as.ppp.matrix")
export("as.ppp.NAobject")
export("as.ppp.ppp")
export("as.ppp.psp")
export("as.ppp.quad")
export("as.psp")
export("as.psp.data.frame")
export("as.psp.default")
export("as.psp.matrix")
export("as.psp.owin")
export("as.psp.psp")
export("as.rectangle")
export("as.solist")
export("as.tess")
export("as.tess.im")
export("as.tess.list")
export("as.tess.owin")
export("as.tess.quadratcount")
export("as.tess.tess")
export("as.tess.tessfun")
export("as.unitname")
export("AsymmDistance.psp")
export("avenndist")
export("bbEngine")
export("bdist.pixels")
export("bdist.points")
export("bdist.tiles")
export("bdry.mask")
export("beachcolourmap")
export("beachcolours")
export("border")
export("boundingbox")
export("bounding.box3")
export("boundingbox.default")
export("boundingbox.im")
export("boundingbox.linnet")
export("boundingbox.list")
export("boundingbox.lpp")
export("boundingbox.owin")
export("boundingbox.ppp")
export("boundingbox.psp")
export("boundingbox.solist")
export("bounding.box.xy")
export("boundingcentre")
export("boundingcentre.owin")
export("boundingcentre.ppp")
export("boundingcircle")
export("boundingcircle.owin")
export("boundingcircle.ppp")
export("boundingradius")
export("boundingradius.owin")
export("boundingradius.ppp")
export("box3")
export("boxx")
export("break.holes")
export("bufftess")
export("by.im")
export("by.ppp")
export("can.be.dfcolumn")
export("can.be.hypercolumn")
export("cartesian")
export("cbind.hyperframe")
export("cellmiddles")
export("centroid.owin")
export("check.arc")
export("checkbigmatrix")          
export("checkfields")          
export("check.finespacing")
export("chop.tess")
export("circumradius")
export("circumradius.owin")
export("circumradius.ppp")
export("circunion")
export("classIgnoringNA")
export("c.layered")
export("clear.simplepanel")
export("clickbox")
export("clickdist")
export("clickpoly")
export("clickppp")
export("clip.infline")
export("clippoly.psp")
export("clip.psp")
export("cliprect.psp")
export("closepairs")
export("closepairs.pp3")
export("closepairs.ppp")
export("closethresh")
export("closetriples")
export("closing")
export("closing.owin")
export("closing.ppp")
export("closing.psp")
export("cobble.xy")
export("cocoEngine")
export("cocoLabels")
export("codetime")
export("coerce.marks.numeric")
export("coerceNAtoObject")
export("col2hex")
export("col.args.to.grey")
export("colourmap")
export("colouroutputs")
export("colouroutputs<-")
export("commonGrid")
export("commonPolyclipArgs")
export("compatible")
export("compatible.im")
export("compatible.unitname")
export("complementarycolour")
export("complement.owin")
export("Complex.im")
export("Complex.imlist")
export("concatxy")
export("conform.imagelist")
export("connected")
export("connected.im")
export("connected.owin")
export("connected.pp3")
export("connected.ppp")
export("connected.tess")
export("contour.funxy")
export("contour.im")
export("contour.imlist")
export("contour.listof")
export("convexhull")
export("convexhull.xy")
export("convexify")
export("convexmetric")
export("convolve.im")
export("coords")
export("coords<-")
export("coords.ppp")
export("coords<-.ppp")
export("coords.ppx")
export("coords<-.ppx")
export("coords.quad")
export("corners")
export("countingweights")
export("covering")
export("crossdist")
export("crossdist.default")
export("crossdist.pp3")
export("crossdist.ppp")
export("crossdist.ppx")
export("crossdist.psp")
export("crossing.psp")
export("crosspairquad")
export("crosspairs")
export("crosspairs.pp3")
export("crosspairs.ppp")
export("cut.im")
export("cut.ppp")
export("default.colourmap")
export("default.dummy")
export("default.image.colours")
export("default.ntile")
export("default.n.tiling")
export("default.symbolmap")
export("default.symbolmap.ppp")
export("delaunay")
export("delaunayDistance")
export("deltametric")
export("dflt.redraw")
export("[.diagramobj")
export("diagramobj")
export("diameter")
export("diameter.box3")
export("diameter.boxx")
export("diameter.owin")
export("dilated.areas")
export("dilation")
export("dilationAny")
export("dilation.owin")
export("dilation.ppp")
export("dilation.psp")
export("dim.hyperframe")               
export("dim.im")               
export("dimnames.hyperframe")
export("dimnames<-.hyperframe")
export("dim.owin")
export("dirichlet")
export("dirichletAreas")
export("dirichletEdges")
export("dirichletVertices")
export("dirichletWeights")
export("disc")
export("discpartarea")
export("discretise")
export("discs")
export("dist2dpath")
export("distfun")
export("distfun.owin")
export("distfun.ppp")
export("distfun.psp")
export("distmap")
export("distmap.owin")
export("distmap.ppp")
export("distmap.psp")
export("do.as.im")
export("do.call.plotfun")
export("domain")
export("domain.distfun")
export("domain.funxy")
export("domain.im")
export("domain.layered")
export("domain.nnfun")
export("domain.pp3")
export("domain.ppp")
export("domain.ppx")
export("domain.psp")
export("domain.quad")
export("domain.quadratcount")
export("domain.tess")
export("drawSignedPoly")
export("dummytilecentroids")        
export("duplicated.ppp")
export("duplicated.ppx")
export("edges")
export("edges2triangles")
export("edges2vees")
export("edit.hyperframe")
export("edit.im")
export("edit.ppp")
export("edit.psp")
export("elementOrNA")
export("ellipse")
export("emptywindow")
export("endpoints.psp")
export("ensureModelSupport")
export("equalpairs.quad")          
export("equalsfun.quad")          
export("equals.quad")          
export("eroded.areas")
export("eroded.volumes")
export("eroded.volumes.box3")
export("eroded.volumes.boxx")
export("erodemask")
export("erosion")
export("erosionAny")
export("erosion.owin")
export("erosion.ppp")
export("erosion.psp")
export("eval.im")
export("even.breaks.owin")
export("exactdt")              
export("exactPdt")
export("existsSpatstatVariable")
export("expandSpecialLists")
export("extrapolate.psp")
export("fakemaintitle")
export("fardist")
export("fardist.owin")
export("fardist.ppp")
export("FeretBox")
export("FeretDiamFun")
export("fft2D")
export("fftwAvailable")
export("fillholes.owin")
export("fillNA")
export("flipxy")
export("flipxy.diagramobj")
export("flipxy.distfun")
export("flipxy.im")
export("flipxy.infline")
export("flipxy.layered")
export("flipxy.owin")
export("flipxypolygon")
export("flipxy.ppp")
export("flipxy.psp")
export("flipxy.tess") 
export("format.numberwithunit")
export("fourierbasis")
export("fourierbasisraw")
export("Frame")
export("Frame<-")
export("framebottomleft")
export("Frame.default")
export("Frame<-.default")
export("framedist.pixels")
export("framedist.pixels")
export("Frame<-.im")
export("Frame.NAobject")
export("Frame<-.owin")
export("Frame<-.ppp")
export("funxy")
export("gammabreaks")
export("genericNNdistBy")
export("getfields")
export("getlastshift")
export("getrasterinfo")
export("getSpatstatVariable")
export("grid1index")
export("gridcenters")
export("gridcentres")
export("gridindex")            
export("gridweights")
export("grow.box3")
export("grow.boxx")
export("grow.mask")
export("grow.rectangle")
export("grow.simplepanel")
export("Halton")
export("Hammersley")
export("handle.r.b.args")
export("harmonise")
export("harmonise.im")
export("harmoniseLevels")
export("harmonise.owin")
export("harmonise.unitname")
export("harmonize")
export("harmonize.im")
export("harmonize.owin")
export("harmonize.unitname")
export("has.close")
export("has.close.default")
export("has.close.pp3")
export("has.close.ppp")
export("head.hyperframe")
export("head.ppp")
export("head.ppx")
export("head.psp")
export("head.tess")
export("hexagon")
export("hexgrid")
export("hextess")
export("hist.funxy")
export("hist.im")
export("hsvim")
export("hsvNA")
export("[.hyperframe")
export("[<-.hyperframe")
export("[[.hyperframe")
export("[[<-.hyperframe")
export("hyperframe")
export("identify.ppp")
export("identify.psp")
export("identify.tess")
export("idorempty")
export("[.im")
export("[<-.im")
export("im")
export("image.im")
export("image.imlist")
export("image.listof")
export("imagelistOp")
export("imageOp")
export("im.apply")
export("ImApplyEngine")
export("imcov")
export("incircle")
export("infline")
export("inpoint")
export("inradius")
export("inside.arc")
export("inside.boxx")
export("inside.owin")
export("integral.im")
export("integral.tessfun")
export("intensity")
export("intensity.ppp")
export("intensity.ppx")
export("intensity.psp")
export("intensity.quadratcount")
export("intensity.splitppp")
export("interp.colourmap")
export("interp.colours")
export("interp.im")
export("interpretAsOrigin")
export("intersect.boxx")
export("intersect.owin")
export("intersect.tess")
export("intX.owin")     
export("intX.xypolygon")     
export("intY.owin")     
export("intY.xypolygon")
export("invokeColourmapRule")
export("invoke.metric")
export("invoke.symbolmap")
export("is.boxx")
export("is.col.argname")
export("is.colour")
export("is.connected")
export("is.connected.default")
export("is.connected.ppp")
export("is.convex")
export("is.data")
export("is.empty")
export("is.empty.default")
export("is.empty.owin")
export("is.empty.ppp")
export("is.empty.psp")
export("is.fv")
export("is.grey")
export("is.hyperframe")
export("is.im")
export("is.imlist")
export("is.infline")
export("is.linim")
export("is.linnet")
export("is.lpp")
export("is.marked")
export("is.marked.default")    
export("is.marked.NAobject")    
export("is.marked.ppp")
export("is.marked.psp")    
export("is.marked.quad")
export("is.mask")
export("is.multitype")
export("is.multitype.default")    
export("is.multitype.NAobject")    
export("is.multitype.ppp")
export("is.multitype.quad")    
export("is.na.hyperframe")
export("is.NAobject")
export("is.na.solist")
export("is.owin")
export("is.polygonal")
export("is.pp3")
export("is.ppp")
export("is.ppplist")
export("is.ppx")
export("is.psp")
export("is.quad")
export("is.rectangle")
export("is.sob")
export("is.solist")
export("is.subset.owin")
export("is.tess")
export("is.vanilla")
export("[.layered")
export("[<-.layered")
export("[[<-.layered")
export("layered")
export("layerplotargs")
export("layerplotargs<-")
export("layout.boxes")
export("lengths_psp")
export("levelsAsFactor")
export("levelset")
export("levelset.default")
export("levelset.distfun")
export("levelset.im")
export("levelset.tessfun")
export("levels.im")
export("levels<-.im")
export("[<-.listof")
export("listof")
export("logi.dummy")
export("lookup.im")
export("lut")
export("majorminorversion")
export("makeunitname")
export("markappend")
export("markappendop")
export("markcbind")
export("markformat")
export("markformat.default")
export("markformat.ppp")
export("markformat.ppx")
export("markformat.psp")
export("markreplicateop")
export("marks")
export("marks<-")           
export("mark.scale.default")
export("marks.default")           
export("marks.NAobject")           
export("marks<-.NAobject")           
export("markspace.integral")
export("marks.ppp")           
export("marks<-.ppp")           
export("marks.ppx")           
export("marks<-.ppx")           
export("marks.psp")
export("marks<-.psp")
export("marks.quad")           
export("markstat")
export("marks.tess")
export("marks<-.tess")
export("marksubset")
export("marksubsetop")
export("markvaluetype")
export("mask2df")
export("matchingdist")
export("Math.im")
export("Math.imlist")
export("maxFeretDiam")
export("maxflow")
export("maxnndist")
export("mean.im")
export("meanX.owin")
export("meanY.owin")
export("median.im")
export("mergeLevels")
export("mergeTiles")
export("mergeTiles.tess")
export("midpoints.psp")
export("minFeretDiam")
export("MinimalTess")
export("MinkowskiSum")
export("MinkowskiSumInternal")
export("minnndist")
export("multiplicity")
export("multiplicity.data.frame")
export("multiplicity.default")
export("multiplicityNumeric")
export("multiplicity.ppp")
export("multiplicity.ppx")
export("multiply.only.finite.entries")
export("na.handle.im")
export("names.hyperframe")
export("names<-.hyperframe")
export("NAobject")
export("nearest.pixel")
export("nearest.raster.point")
export("nearestsegment")
export("nearest.valid.pixel")
export("nearestValue")
export("needpackage")
export("nestsplit")
export("nncross")
export("nncross.default")
export("nncross.pp3")
export("nncross.ppp")
export("nncross.ppx")
export("nndist")
export("nndist.default")
export("nndist.pp3")
export("nndist.ppp")
export("nndist.ppx")
export("nndist.psp")
export("nnfun")
export("nnfun.ppp")
export("nnfun.psp")
export("nnmap")
export("nnmark")
export("nnwhich")
export("nnwhich.default")
export("nnwhich.pp3")
export("nnwhich.ppp")
export("nnwhich.ppx")
export("nobjects")
export("nobjects.NAobject")
export("nobjects.ppp")
export("nobjects.ppx")
export("nobjects.psp")
export("nobjects.tess")
export("npoints")
export("npoints.NAobject")
export("npoints.pp3")
export("npoints.ppp")
export("npoints.ppx")
export("npoints.quad")
export("n.quad")
export("nsegments")
export("nsegments.psp")
export("numberwithunit")
export("numeric.columns")
export("nvertices")
export("nvertices.default")
export("nvertices.owin")
export("onearrow")
export("onecolumn")
export("opening")
export("opening.owin")
export("opening.ppp")
export("opening.psp")
export("Ops.im")
export("Ops.imlist")
export("overlap.owin")
export("[.owin")
export("owin")
export("owin2mask")
export("owin2polypath")
export("owinInternalMask")
export("owinInternalPoly")
export("owinInternalRect")
export("owinpoly2mask")
export("owinpolycheck")
export("packupNNdata")
export("padimage")
export("pairdist")
export("pairdist.default")
export("pairdist.pp3")
export("pairdist.ppp")
export("pairdist.ppx")
export("pairdist.psp")
export("paletteindex")
export("paletteindex")
export("param.quad")
export("parbreak")
export("PDtoNN")
export("perimeter")
export("periodify")
export("periodify.owin")
export("periodify.ppp")
export("periodify.psp")
export("perspContour")
export("persp.funxy")
export("persp.im")
export("perspLines")
export("perspPoints")
export("persp.ppp")
export("perspSegments")
export("perspVisible")
export("pHcolour")
export("pHcolourmap")
export("pickoption")
export("pixelcentres")
export("pixellate")
export("pixellate.owin")
export("pixellate.ppp")
export("pixellate.psp")
export("pixelquad")
export("plan.legend.layout")
export("plot3Dpoints")
export("plot.anylist")
export("plot.barplotdata")
export("plot.colourmap")
export("plotEachLayer")
export("plot.funxy")
export("plot.hyperframe")
export("plot.im")
export("plot.imlist")
export("plot.indicfun")
export("plot.infline")
export("plot.layered")
export("plot.listof")
export("plot.NAobject")
export("plot.onearrow")
export("plot.owin")
export("plotPolygonBdry")
export("plot.pp3")
export("plot.ppp")
export("plot.pppmatching")
export("plot.ppx")
export("plot.psp")
export("plot.quad")
export("plot.quadratcount")
export("plot.solist")
export("plot.splitppp")
export("plot.symbolmap")
export("plot.tess")
export("plot.tessfun")
export("plot.textstring")
export("plot.texturemap")
export("plotWidthMap")
export("plot.yardstick")
export("pointgrid")
export("pointsOnLines")
export("pointweights")
export("polarbody")
export("polartess")
export("polytileareaEngine")
export("[.pp3")
export("pp3")
export("ppllengine")
export("[.ppp")
export("[<-.ppp")
export("ppp")
export("pppdist")
export("pppdist.mat")
export("pppdist.prohorov")
export("pppmatching")
export("ppsubset")
export("[.ppx")
export("ppx")
export("prepareTitle")
export("print.anylist")    
export("print.box3")
export("print.boxx")
export("print.colourmap")
export("print.distfun")
export("print.funxy")       
export("print.hyperframe")
export("print.im")
export("print.indicfun")       
export("print.infline")
export("print.layered")
export("print.lut")
export("print.metric")
export("print.metricfun")
export("print.NAobject")
export("print.nnfun")
export("print.numberwithunit")
export("print.onearrow")
export("print.owin")
export("print.pp3")
export("print.ppp")
export("print.pppmatching")
export("print.ppx")
export("print.psp")
export("print.quad")
export("print.simplepanel")
export("print.solist")
export("print.splitppp")
export("print.splitppx")
export("print.summary.distfun")
export("print.summary.funxy")
export("print.summary.hyperframe")
export("print.summary.im")
export("print.summary.listof")
export("print.summary.logiquad")
export("print.summary.lut")
export("print.summary.NAobject")
export("print.summary.owin")
export("print.summary.pp3")
export("print.summary.ppp")
export("print.summary.psp")
export("print.summary.quad")
export("print.summary.solist")
export("print.summary.splitppp")
export("print.summary.splitppx")
export("print.summary.symbolmap")       
export("print.summary.tess")       
export("print.summary.unitname")
export("print.symbolmap")
export("print.tess")
export("print.tessfun")
export("print.textstring")
export("print.texturemap")
export("print.timed")
export("print.unitname")
export("print.yardstick")
export("progressreport")
export("project2segment")
export("project2set")
export("project3Dhom")
export("[.psp")
export("psp")
export("psp2mask")
export("putlastshift")
export("putSpatstatVariable")
export("qtPrepareCoordinate")
export("[.quad")
export("quad")
export("quadratcount")
export("quadratcount.ppp")
export("quadratcount.splitppp")
export("quadrats")
export("quadscheme")
export("quadscheme.logi")
export("quadscheme.replicated")
export("quadscheme.spatial")
export("quantess")
export("quantess.im")
export("quantess.owin")
export("quantess.ppp")
export("quantilefun.im")
export("quantile.im")
export("rasterfilter")
export("rastersample")
export("raster.x")
export("rasterx.im")
export("rasterx.mask")
export("raster.xy")
export("rasterxy.im")
export("rasterxy.mask")
export("raster.y")
export("rastery.im")
export("rastery.mask")
export("rbind.hyperframe")
export("rebound")
export("rebound.im")
export("rebound.owin")
export("rebound.ppp")
export("rebound.psp")
export("recognise.plotcommand.type")
export("recognise.spatstat.type")
export("rectdistmap")
export("rectquadrat.breaks")
export("rectquadrat.countEngine")
export("redraw.simplepanel")
export("reflect")
export("reflect.default")
export("reflect.diagramobj")
export("reflect.distfun")
export("reflect.im")
export("reflect.infline")
export("reflect.layered")
export("reflect.tess") 
export("regularpolygon")
export("relevel.im")
export("relevel.ppp")
export("relevel.ppx")
export("remove.identical.pairs")
export("repair.image.xycoords")
export("repair.old.factor.image")
export("requireversion")
export("rescale")
export("rescale.distfun")
export("rescale.im")
export("rescale.layered")
export("rescale.owin")
export("rescale.ppp")
export("rescale.psp")
export("rescale.unitname")
export("rescue.rectangle")
export("reset.default.image.colours")
export("reset.spatstat.options")
export("resolve.stringsAsFactors")
export("restrict.colourmap")
export("restrict.mask")
export("rev.colourmap")
export("reversePolyclipArgs")
export("rexplode")
export("rexplode.ppp")
export("rgb2hex")
export("rgb2hsva")
export("rgbim")
export("rgbNA")
export("ripras")
export("rjitter")
export("rjitter.ppp")
export("rlinegrid")
export("rotate")
export("rotate.diagramobj")
export("rotate.distfun")
export("rotate.im")
export("rotate.infline")
export("rotate.layered")
export("rotate.owin")
export("rotate.ppp")
export("rotate.psp")
export("rotate.tess") 
export("rotxy")
export("rotxypolygon")
export("rounding.pp3")
export("rounding.ppp")
export("rounding.ppx")
export("round.pp3")
export("round.ppp")
export("round.ppx")
export("row.names.hyperframe")
export("row.names<-.hyperframe")
export("rQuasi")
export("rsyst")
export("ruletextline")
export("runifrect")
export("run.simplepanel")
export("safedeldir")
export("safeDevCapabilities")
export("safelookup")
export("samecolour")
export("scalardilate")
export("scalardilate.breakpts")
export("scalardilate.default")
export("scalardilate.diagramobj")
export("scalardilate.distfun")
export("scalardilate.im")
export("scalardilate.layered")
export("scalardilate.owin")
export("scalardilate.ppp")
export("scalardilate.psp")
export("scalardilate.tess") 
export("scale.boxx")
export("scale.ppx")
export("scaletointerval")
export("scaletointerval.default")
export("scaletointerval.im")
export("scanpp")
export("selfcrossing.psp")
export("selfcut.psp")
export("sessionLibs")
export("setcov")
export("setmarks")
export("setminus.owin")
export("shift")
export("shift.boxx")
export("shift.diagramobj")              
export("shift.distfun")
export("shift.im")
export("shift.infline")
export("shift.layered")
export("shift.owin")
export("shift.ppp")
export("shift.ppx")
export("shift.psp")
export("shift.quadratcount")              
export("shift.tess") 
export("shiftxy")              
export("shiftxypolygon")              
export("shortside")
export("shortside.box3")
export("shortside.boxx")
export("shortside.owin")
export("sidelengths")
export("sidelengths.box3")
export("sidelengths.boxx")
export("sidelengths.owin")
export("simplepanel")
export("simplify.owin")
export("simulationresult")
export("smudge")
export("solapply")
export("[.solist")
export("[<-.solist")
export("[[<-.solist")
export("solist")
export("solutionset")
export("sort.im")
export("spatdim")
export("spatstat.deldir.setopt")
export("spatstatDiagnostic")
export("spatstat.options")
export("spatstat.xy.coords")
export("split.hyperframe")
export("split<-.hyperframe")
export("split.im")
export("[.splitppp")
export("[<-.splitppp")
export("split.ppp")
export("split<-.ppp")
export("[.splitppx")
export("[<-.splitppx")
export("split.ppx")
export("spokes")
export("square")
export("store.versionstring.spatstat")
export("stratrand")
export("str.hyperframe")
export("subset.hyperframe")
export("subset.pp3")
export("subset.ppp")
export("subset.ppx")
export("subset.psp")
export("summary.anylist")
export("summary.distfun")
export("summary.funxy")
export("summary.hyperframe")
export("summary.im")
export("Summary.im")
export("Summary.imlist")
export("summary.listof")
export("summary.logiquad")
export("summary.lut")
export("summary.metric")
export("summary.NAobject")
export("summary.owin")
export("summary.pp3")
export("summary.ppp")
export("summary.pppmatching")
export("summary.ppx")
export("summary.psp")
export("summary.quad")
export("summary.solist")
export("summary.splitppp")
export("summary.splitppx")
export("summary.symbolmap")
export("summary.tess")
export("summary.unitname")
export("superimpose")
export("superimpose.default")
export("superimposeMarks")
export("superimpose.ppp")
export("superimpose.ppplist")
export("superimpose.psp")
export("superimpose.splitppp")
export("SupportFun")
export("symbolmap")
export("symbolmapdomain")
export("symbolmapparnames")
export("symbolmaptype")
export("symbol.sizes.default")
export("tail.hyperframe")
export("tail.ppp")
export("tail.ppx")
export("tail.psp")
export("tail.tess")
export("[.tess")
export("[<-.tess")
export("tess")
export("tessfunvalues")
export("test.crossing.psp")
export("test.selfcrossing.psp")
export("text.ppp")
export("text.psp")
export("textstring")
export("texturemap")
export("textureplot")
export("thickSegments")
export("tile.areas")
export("tile.centroids")
export("tileindex")
export("tilenames")
export("tilenames<-")
export("tilenames.tess")
export("tilenames<-.tess")
export("tiles")
export("tiles.empty")
export("timed")
export("timeTaken")
export("to.grey")
export("to.opaque")
export("to.saturated")
export("to.transparent")
export("transmat")
export("trianglediameters")
export("triangulate.owin")
export("trim.mask")
export("trim.rectangle")
export("tweak.closepairs")
export("tweak.colourmap")
export("union.owin")
export("union.quad")
export("uniquemap.lpp")
export("uniquemap.ppp")
export("uniquemap.ppx")
export("unique.ppp")
export("unique.ppx")
export("unitname")
export("unitname<-")
export("unitname.box3")
export("unitname<-.box3")
export("unitname.boxx")
export("unitname<-.boxx")
export("unitname.default")
export("unitname<-.default")
export("unitname.im")
export("unitname<-.im")
export("unitname.owin")
export("unitname<-.owin")
export("unitname.pp3")
export("unitname<-.pp3")
export("unitname.ppp")
export("unitname<-.ppp")
export("unitname.ppx")
export("unitname<-.ppx")
export("unitname.psp")
export("unitname<-.psp")
export("unitname.quad")
export("unitname<-.quad")
export("unitname.tess")
export("unitname<-.tess")
export("unit.square")
export("unmark")
export("unmark.ppp")
export("unmark.ppx")
export("unmark.psp")
export("unmark.splitppp")
export("unmark.tess")
export("unstackFilter")
export("unstack.layered")
export("unstack.ppp")
export("unstack.psp")
export("unstack.solist")
export("unstack.tess")
export("update.im")
export("update.symbolmap")
export("validate.mask")        
export("validate.quad")        
export("validradius")
export("vdCorput")
export("venn.tess")
export("verifyclass")
export("versioncurrency.spatstat")
export("versionstring.spatstat")
export("vertices")
export("vertices.owin")
export("veryunique")
export("volume")
export("volume.box3")
export("volume.boxx")
export("volume.owin")
export("voronoiFlower")
export("warn.no.metric.support")
export("warn.once")
export("warn.unsupported.args")
export("waxlyrical")
export("where.max")
export("where.min")
export("whichhalfplane")
export("Window")
export("Window<-")
export("Window.distfun")
export("Window.funxy")
export("Window.im")
export("Window<-.im")
export("Window.layered")
export("Window.NAobject")
export("Window.nnfun")
export("Window.ppp")
export("Window<-.ppp")
export("Window.psp")
export("Window<-.psp")
export("Window.quad")
export("Window<-.quad")
export("Window.quadratcount")
export("Window.tess")
export("with.hyperframe")
export("w.quad")               
export("wrangle2image")
export("XDtoNN")
export("x.quad")
export("xtfrm.im")
export("xy.grid")
export("xypoly2owin")
export("xypolycoco")
export("xypolycomponents")
export("xypolygon2psp")
export("xypolyNesting")
export("xypolyselfint")
export("xypolysub2owin")
export("yardstick")
export("y.quad")
export("zapsmall.im")
export("ZeroValue")
export("ZeroValue.im")

# .......  Special cases ........... 
export("%(+)%")
export("%(-)%")
export("%mapp%")
export("%mark%")
export("%mrep%")
export("%msub%")
export("%unit%")
S3method("Complex", "im")
S3method("Complex", "imlist")
S3method("Math", "im")
S3method("Math", "imlist")
S3method("mean", "im")
S3method("median", "im")
S3method("Ops", "im")
S3method("Ops", "imlist")
S3method("Summary", "im")
S3method("Summary", "imlist")
# .......  End of special cases  ... 

# ......................................... 
# Automatically generated list of S3 methods
# ......................................... 
  S3method("$", "hyperframe")
S3method("affine", "diagramobj")
S3method("affine", "distfun")
  S3method("affine", "im")
  S3method("affine", "layered")
  S3method("affine", "owin")
 S3method("affine", "ppp")
 S3method("affine", "psp")
  S3method("affine", "tess")
 S3method("anyDuplicated", "ppp")
 S3method("anyDuplicated", "ppx")
  S3method("[", "anylist")
  S3method("anyNA", "im")
 S3method("area", "default")
 S3method("area", "owin")
  S3method("as.array", "im")
S3method("as.character", "unitname")
  S3method("as.colourmap", "colourmap")
  S3method("as.colourmap", "symbolmap")
S3method("as.data.frame", "hyperframe")
  S3method("as.data.frame", "im")
S3method("as.data.frame", "NAobject")
S3method("as.data.frame", "owin")
S3method("as.data.frame", "ppp")
S3method("as.data.frame", "ppplist")
S3method("as.data.frame", "ppx")
S3method("as.data.frame", "psp")
S3method("as.data.frame", "tess")
S3method("as.double", "im")
 S3method("as.function", "im")
 S3method("as.function", "owin")
  S3method("as.function", "tess")
S3method("as.hyperframe", "anylist")
S3method("as.hyperframe", "data.frame")
S3method("as.hyperframe", "default")
S3method("as.hyperframe", "hyperframe")
S3method("as.hyperframe", "listof")
S3method("as.hyperframe", "ppx")
  S3method("as.im", "data.frame")
  S3method("as.im", "default")
  S3method("as.im", "distfun")
  S3method("as.im", "expression")
  S3method("as.im", "function")
  S3method("as.im", "funxy")
  S3method("as.im", "im")
  S3method("as.im", "matrix")
S3method("as.im", "NAobject")
  S3method("as.im", "nnfun")
  S3method("as.im", "owin")
S3method("as.im", "ppp")
  S3method("as.im", "tess")
 S3method("as.layered", "default")
 S3method("as.layered", "listof")
 S3method("as.layered", "ppp")
 S3method("as.layered", "solist")
 S3method("as.layered", "splitppp")
S3method("as.list", "hyperframe")
  S3method("as.matrix", "im")
  S3method("as.matrix", "owin")
S3method("as.matrix", "ppx")
 S3method("as.owin", "boxx")
 S3method("as.owin", "data.frame")
 S3method("as.owin", "default")
 S3method("as.owin", "distfun")
 S3method("as.owin", "funxy")
 S3method("as.owin", "im")
 S3method("as.owin", "layered")
S3method("as.owin", "NAobject")
 S3method("as.owin", "nnfun")
 S3method("as.owin", "owin")
 S3method("as.owin", "ppp")
 S3method("as.owin", "psp")
 S3method("as.owin", "quad")
 S3method("as.owin", "quadratcount")
 S3method("as.owin", "tess")
  S3method("as.ppp", "data.frame")
  S3method("as.ppp", "default")
  S3method("as.ppp", "matrix")
S3method("as.ppp", "NAobject")
  S3method("as.ppp", "ppp")
  S3method("as.ppp", "psp")
  S3method("as.ppp", "quad")
  S3method("as.psp", "data.frame")
  S3method("as.psp", "default")
  S3method("as.psp", "matrix")
S3method("as.psp", "owin")
  S3method("as.psp", "psp")
 S3method("as.tess", "im")
 S3method("as.tess", "list")
 S3method("as.tess", "owin")
 S3method("as.tess", "quadratcount")
 S3method("as.tess", "tess")
S3method("as.tess", "tessfun")
S3method("boundingbox", "default")
S3method("boundingbox", "im")
S3method("boundingbox", "linnet")
S3method("boundingbox", "list")
S3method("boundingbox", "lpp")
S3method("boundingbox", "owin")
S3method("boundingbox", "ppp")
S3method("boundingbox", "psp")
S3method("boundingbox", "solist")
S3method("boundingcentre", "owin")
S3method("boundingcentre", "ppp")
S3method("boundingcircle", "owin")
S3method("boundingcircle", "ppp")
S3method("boundingradius", "owin")
S3method("boundingradius", "ppp")
S3method("by", "im")
 S3method("by", "ppp")
S3method("cbind", "hyperframe")
S3method("circumradius", "owin")
S3method("circumradius", "ppp")
  S3method("c", "layered")
S3method("closepairs", "pp3")
S3method("closepairs", "ppp")
 S3method("closing", "owin")
 S3method("closing", "ppp")
 S3method("closing", "psp")
 S3method("compatible", "im")
  S3method("compatible", "unitname")
S3method("connected", "im")
S3method("connected", "owin")
S3method("connected", "pp3")
S3method("connected", "ppp")
S3method("connected", "tess")
S3method("contour", "funxy")
   S3method("contour", "im")
  S3method("contour", "imlist")
  S3method("contour", "listof")
  S3method("coords", "ppp")
  S3method("coords", "ppx")
  S3method("coords", "quad")
  S3method("crossdist", "default")
  S3method("crossdist", "pp3")
  S3method("crossdist", "ppp")
  S3method("crossdist", "ppx")
  S3method("crossdist", "psp")
S3method("crosspairs", "pp3")
S3method("crosspairs", "ppp")
  S3method("cut", "im")
  S3method("cut", "ppp")
S3method("default.symbolmap", "ppp")
S3method("[", "diagramobj")
S3method("diameter", "box3")
S3method("diameter", "boxx")
 S3method("diameter", "owin")
 S3method("dilation", "owin")
 S3method("dilation", "ppp")
 S3method("dilation", "psp")
S3method("dim", "hyperframe")
S3method("dim", "im")
S3method("dimnames", "hyperframe")
S3method("dim", "owin")
  S3method("distfun", "owin")
  S3method("distfun", "ppp")
  S3method("distfun", "psp")
  S3method("distmap", "owin")
  S3method("distmap", "ppp")
  S3method("distmap", "psp")
 S3method("domain", "distfun")
 S3method("domain", "funxy")
   S3method("domain", "im")
 S3method("domain", "layered")
 S3method("domain", "nnfun")
   S3method("domain", "pp3")
   S3method("domain", "ppp")
   S3method("domain", "ppx")
   S3method("domain", "psp")
 S3method("domain", "quad")
 S3method("domain", "quadratcount")
 S3method("domain", "tess")
 S3method("duplicated", "ppp")
 S3method("duplicated", "ppx")
S3method("edit", "hyperframe")
S3method("edit", "im")
S3method("edit", "ppp")
S3method("edit", "psp")
S3method("eroded.volumes", "box3")
S3method("eroded.volumes", "boxx")
 S3method("erosion", "owin")
 S3method("erosion", "ppp")
 S3method("erosion", "psp")
  S3method("fardist", "owin")
  S3method("fardist", "ppp")
S3method("flipxy", "diagramobj")
S3method("flipxy", "distfun")
 S3method("flipxy", "im")
S3method("flipxy", "infline")
  S3method("flipxy", "layered")
 S3method("flipxy", "owin")
 S3method("flipxy", "ppp")
 S3method("flipxy", "psp")
  S3method("flipxy", "tess")
S3method("format", "numberwithunit")
   S3method("Frame", "default")
S3method("Frame", "NAobject")
S3method("harmonise", "im")
S3method("harmonise", "owin")
  S3method("harmonise", "unitname")
S3method("harmonize", "im")
S3method("harmonize", "owin")
  S3method("harmonize", "unitname")
  S3method("has.close", "default")
  S3method("has.close", "pp3")
  S3method("has.close", "ppp")
S3method("head", "hyperframe")
  S3method("head", "ppp")
  S3method("head", "ppx")
  S3method("head", "psp")
  S3method("head", "tess")
  S3method("hist", "funxy")
  S3method("hist", "im")
  S3method("[", "hyperframe")
  S3method("[[", "hyperframe")
  S3method("identify", "ppp")
  S3method("identify", "psp")
  S3method("identify", "tess")
  S3method("[", "im")
  S3method("image", "im")
  S3method("image", "imlist")
  S3method("image", "listof")
S3method("integral", "im")
 S3method("integral", "tessfun")
S3method("intensity", "ppp")
  S3method("intensity", "ppx")
S3method("intensity", "psp")
S3method("intensity", "quadratcount")
S3method("intensity", "splitppp")
S3method("is.connected", "default")
S3method("is.connected", "ppp")
S3method("is.empty", "default")
S3method("is.empty", "owin")
S3method("is.empty", "ppp")
S3method("is.empty", "psp")
S3method("is.marked", "default")
S3method("is.marked", "NAobject")
  S3method("is.marked", "ppp")
S3method("is.marked", "psp")
S3method("is.marked", "quad")
S3method("is.multitype", "default")
S3method("is.multitype", "NAobject")
  S3method("is.multitype", "ppp")
S3method("is.multitype", "quad")
S3method("is.na", "hyperframe")
S3method("is.na", "solist")
  S3method("[", "layered")
S3method("levelset", "default")
S3method("levelset", "distfun")
  S3method("levelset", "im")
S3method("levelset", "tessfun")
S3method("levels", "im")
S3method("markformat", "default")
S3method("markformat", "ppp")
S3method("markformat", "ppx")
S3method("markformat", "psp")
S3method("marks", "default")
S3method("marks", "NAobject")
S3method("marks", "ppp")
S3method("marks", "ppx")
S3method("marks", "psp")
S3method("marks", "quad")
S3method("marks", "tess")
 S3method("mergeTiles", "tess")
 S3method("multiplicity", "data.frame")
 S3method("multiplicity", "default")
 S3method("multiplicity", "ppp")
 S3method("multiplicity", "ppx")
S3method("names", "hyperframe")
  S3method("nncross", "default")
  S3method("nncross", "pp3")
  S3method("nncross", "ppp")
  S3method("nncross", "ppx")
  S3method("nndist", "default")
  S3method("nndist", "pp3")
  S3method("nndist", "ppp")
  S3method("nndist", "ppx")
  S3method("nndist", "psp")
  S3method("nnfun", "ppp")
  S3method("nnfun", "psp")
  S3method("nnwhich", "default")
  S3method("nnwhich", "pp3")
  S3method("nnwhich", "ppp")
  S3method("nnwhich", "ppx")
S3method("nobjects", "NAobject")
  S3method("nobjects", "ppp")
  S3method("nobjects", "ppx")
  S3method("nobjects", "psp")
  S3method("nobjects", "tess")
S3method("npoints", "NAobject")
  S3method("npoints", "pp3")
  S3method("npoints", "ppp")
  S3method("npoints", "ppx")
S3method("npoints", "quad")
   S3method("nsegments", "psp")
  S3method("nvertices", "default")
  S3method("nvertices", "owin")
 S3method("opening", "owin")
 S3method("opening", "ppp")
 S3method("opening", "psp")
  S3method("[", "owin")
  S3method("pairdist", "default")
  S3method("pairdist", "pp3")
  S3method("pairdist", "ppp")
  S3method("pairdist", "ppx")
  S3method("pairdist", "psp")
S3method("periodify", "owin")
S3method("periodify", "ppp")
S3method("periodify", "psp")
S3method("persp", "funxy")
  S3method("persp", "im")
  S3method("persp", "ppp")
S3method("pixellate", "owin")
S3method("pixellate", "ppp")
S3method("pixellate", "psp")
  S3method("plot", "anylist")
S3method("plot", "barplotdata")
S3method("plot", "colourmap")
S3method("plot", "funxy")
   S3method("plot", "hyperframe")
  S3method("plot", "im")
  S3method("plot", "imlist")
S3method("plot", "indicfun")
S3method("plot", "infline")
S3method("plot", "layered")
  S3method("plot", "listof")
S3method("plot", "NAobject")
 S3method("plot", "onearrow")
 S3method("plot", "owin")
 S3method("plot", "pp3")
 S3method("plot", "ppp")
S3method("plot", "pppmatching")
S3method("plot", "ppx")
  S3method("plot", "psp")
  S3method("plot", "quad")
S3method("plot", "quadratcount")
  S3method("plot", "solist")
  S3method("plot", "splitppp")
S3method("plot", "symbolmap")
  S3method("plot", "tess")
S3method("plot", "tessfun")
S3method("plot", "textstring")
 S3method("plot", "texturemap")
 S3method("plot", "yardstick")
S3method("[", "pp3")
  S3method("[", "ppp")
  S3method("[", "ppx")
S3method("print", "anylist")
  S3method("print", "box3")
  S3method("print", "boxx")
S3method("print", "colourmap")
S3method("print", "distfun")
S3method("print", "funxy")
S3method("print", "hyperframe")
  S3method("print", "im")
S3method("print", "indicfun")
S3method("print", "infline")
S3method("print", "layered")
S3method("print", "lut")
S3method("print", "metric")
S3method("print", "metricfun")
S3method("print", "NAobject")
S3method("print", "nnfun")
S3method("print", "numberwithunit")
S3method("print", "onearrow")
  S3method("print", "owin")
  S3method("print", "pp3")
  S3method("print", "ppp")
S3method("print", "pppmatching")
S3method("print", "ppx")
  S3method("print", "psp")
 S3method("print", "quad")
S3method("print", "simplepanel")
S3method("print", "solist")
S3method("print", "splitppp")
S3method("print", "splitppx")
S3method("print", "summary.distfun")
S3method("print", "summary.funxy")
S3method("print", "summary.hyperframe")
  S3method("print", "summary.im")
S3method("print", "summary.listof")
S3method("print", "summary.logiquad")
S3method("print", "summary.lut")
S3method("print", "summary.NAobject")
S3method("print", "summary.owin")
  S3method("print", "summary.pp3")
S3method("print", "summary.ppp")
S3method("print", "summary.psp")
  S3method("print", "summary.quad")
S3method("print", "summary.solist")
S3method("print", "summary.splitppp")
S3method("print", "summary.splitppx")
S3method("print", "summary.symbolmap")
S3method("print", "summary.tess")
S3method("print", "summary.unitname")
S3method("print", "symbolmap")
S3method("print", "tess")
S3method("print", "tessfun")
S3method("print", "textstring")
S3method("print", "texturemap")
S3method("print", "timed")
  S3method("print", "unitname")
S3method("print", "yardstick")
  S3method("[", "psp")
  S3method("[", "quad")
  S3method("quadratcount", "ppp")
  S3method("quadratcount", "splitppp")
S3method("quantess", "im")
S3method("quantess", "owin")
S3method("quantess", "ppp")
  S3method("quantilefun", "im")
S3method("quantile", "im")
S3method("rbind", "hyperframe")
S3method("rebound", "im")
S3method("rebound", "owin")
S3method("rebound", "ppp")
S3method("rebound", "psp")
  S3method("reflect", "default")
S3method("reflect", "diagramobj")
S3method("reflect", "distfun")
  S3method("reflect", "im")
S3method("reflect", "infline")
  S3method("reflect", "layered")
  S3method("reflect", "tess")
  S3method("relevel", "im")
  S3method("relevel", "ppp")
  S3method("relevel", "ppx")
S3method("rescale", "distfun")
 S3method("rescale", "im")
  S3method("rescale", "layered")
  S3method("rescale", "owin")
 S3method("rescale", "ppp")
 S3method("rescale", "psp")
  S3method("rescale", "unitname")
  S3method("rev", "colourmap")
S3method("rexplode", "ppp")
S3method("rjitter", "ppp")
S3method("rotate", "diagramobj")
S3method("rotate", "distfun")
 S3method("rotate", "im")
S3method("rotate", "infline")
  S3method("rotate", "layered")
 S3method("rotate", "owin")
 S3method("rotate", "ppp")
 S3method("rotate", "psp")
  S3method("rotate", "tess")
S3method("rounding", "pp3")
S3method("rounding", "ppp")
S3method("rounding", "ppx")
S3method("round", "pp3")
S3method("round", "ppp")
S3method("round", "ppx")
S3method("row.names", "hyperframe")
S3method("scalardilate", "breakpts")
  S3method("scalardilate", "default")
S3method("scalardilate", "diagramobj")
S3method("scalardilate", "distfun")
  S3method("scalardilate", "im")
  S3method("scalardilate", "layered")
  S3method("scalardilate", "owin")
  S3method("scalardilate", "ppp")
  S3method("scalardilate", "psp")
  S3method("scalardilate", "tess")
  S3method("scale", "boxx")
S3method("scale", "ppx")
  S3method("scaletointerval", "default")
  S3method("scaletointerval", "im")
 S3method("shift", "boxx")
S3method("shift", "diagramobj")
S3method("shift", "distfun")
 S3method("shift", "im")
S3method("shift", "infline")
  S3method("shift", "layered")
 S3method("shift", "owin")
 S3method("shift", "ppp")
 S3method("shift", "ppx")
 S3method("shift", "psp")
S3method("shift", "quadratcount")
  S3method("shift", "tess")
S3method("shortside", "box3")
S3method("shortside", "boxx")
 S3method("shortside", "owin")
S3method("sidelengths", "box3")
S3method("sidelengths", "boxx")
 S3method("sidelengths", "owin")
  S3method("[", "solist")
S3method("sort", "im")
  S3method("split", "hyperframe")
S3method("split", "im")
  S3method("[", "splitppp")
  S3method("split", "ppp")
  S3method("split", "ppx")
S3method("[", "splitppx")
S3method("str", "hyperframe")
S3method("subset", "hyperframe")
S3method("subset", "pp3")
S3method("subset", "ppp")
S3method("subset", "ppx")
S3method("subset", "psp")
 S3method("summary", "anylist")
  S3method("summary", "distfun")
  S3method("summary", "funxy")
S3method("summary", "hyperframe")
  S3method("summary", "im")
 S3method("summary", "listof")
S3method("summary", "logiquad")
S3method("summary", "lut")
S3method("summary", "metric")
S3method("summary", "NAobject")
  S3method("summary", "owin")
  S3method("summary", "pp3")
 S3method("summary", "ppp")
S3method("summary", "pppmatching")
S3method("summary", "ppx")
  S3method("summary", "psp")
  S3method("summary", "quad")
 S3method("summary", "solist")
 S3method("summary", "splitppp")
S3method("summary", "splitppx")
S3method("summary", "symbolmap")
S3method("summary", "tess")
  S3method("summary", "unitname")
  S3method("superimpose", "default")
  S3method("superimpose", "ppp")
  S3method("superimpose", "ppplist")
  S3method("superimpose", "psp")
  S3method("superimpose", "splitppp")
S3method("tail", "hyperframe")
  S3method("tail", "ppp")
  S3method("tail", "ppx")
  S3method("tail", "psp")
  S3method("tail", "tess")
  S3method("[", "tess")
S3method("text", "ppp")
S3method("text", "psp")
S3method("tilenames", "tess")
S3method("uniquemap", "lpp")
S3method("uniquemap", "ppp")
S3method("uniquemap", "ppx")
 S3method("unique", "ppp")
 S3method("unique", "ppx")
  S3method("unitname", "box3")
  S3method("unitname", "boxx")
S3method("unitname", "default")
S3method("unitname", "im")
S3method("unitname", "owin")
  S3method("unitname", "pp3")
S3method("unitname", "ppp")
S3method("unitname", "ppx")
S3method("unitname", "psp")
S3method("unitname", "quad")
S3method("unitname", "tess")
 S3method("unmark", "ppp")
 S3method("unmark", "ppx")
 S3method("unmark", "psp")
 S3method("unmark", "splitppp")
S3method("unmark", "tess")
S3method("unstack", "layered")
S3method("unstack", "ppp")
S3method("unstack", "psp")
S3method("unstack", "solist")
S3method("unstack", "tess")
S3method("update", "im")
S3method("update", "symbolmap")
 S3method("vertices", "owin")
S3method("volume", "box3")
S3method("volume", "boxx")
 S3method("volume", "owin")
 S3method("Window", "distfun")
 S3method("Window", "funxy")
   S3method("Window", "im")
 S3method("Window", "layered")
S3method("Window", "NAobject")
 S3method("Window", "nnfun")
   S3method("Window", "ppp")
   S3method("Window", "psp")
   S3method("Window", "quad")
 S3method("Window", "quadratcount")
 S3method("Window", "tess")
S3method("with", "hyperframe")
S3method("xtfrm", "im")
S3method("ZeroValue", "im")
# ......................................... 
#      Assignment methods                   
# ......................................... 
  S3method("$<-", "hyperframe")
  S3method("[<-", "anylist")
  S3method("coords<-", "ppp")
  S3method("coords<-", "ppx")
S3method("dimnames<-", "hyperframe")
   S3method("Frame<-", "default")
   S3method("Frame<-", "im")
   S3method("Frame<-", "owin")
   S3method("Frame<-", "ppp")
  S3method("[<-", "hyperframe")
  S3method("[[<-", "hyperframe")
  S3method("[<-", "im")
  S3method("[<-", "layered")
  S3method("[[<-", "layered")
S3method("levels<-", "im")
  S3method("[<-", "listof")
S3method("marks<-", "NAobject")
S3method("marks<-", "ppp")
S3method("marks<-", "ppx")
S3method("marks<-", "psp")
S3method("marks<-", "tess")
S3method("names<-", "hyperframe")
  S3method("[<-", "ppp")
S3method("row.names<-", "hyperframe")
  S3method("[<-", "solist")
  S3method("[[<-", "solist")
  S3method("split<-", "hyperframe")
  S3method("[<-", "splitppp")
  S3method("split<-", "ppp")
S3method("[<-", "splitppx")
  S3method("[<-", "tess")
S3method("tilenames<-", "tess")
  S3method("unitname<-", "box3")
  S3method("unitname<-", "boxx")
S3method("unitname<-", "default")
S3method("unitname<-", "im")
S3method("unitname<-", "owin")
  S3method("unitname<-", "pp3")
S3method("unitname<-", "ppp")
S3method("unitname<-", "ppx")
S3method("unitname<-", "psp")
S3method("unitname<-", "quad")
S3method("unitname<-", "tess")
   S3method("Window<-", "im")
   S3method("Window<-", "ppp")
   S3method("Window<-", "psp")
   S3method("Window<-", "quad")
# ......................................... 
#      End of methods                       
# ......................................... 
